/*
 Ce sketch illustre le filtrage par moyenne glissante et par filtre récursif à réponse impulsionnelle infinie ou filtre IIR.
 Le résultat est à observer dans le traceur série (menu "Outils").
 Le programme trace un signal sinusoïdal pur et bruité, puis applique les filtres sur le signal bruité.
 */
#include <Gaussian.h>                   // Bibliothèque permettant de générer des valeurs qui suivent une distribution gaussienne afin d'ajouter un bruit pseudo aléatoire au signal sinusoïdal
Gaussian myGaussian = Gaussian(0, 100); // Création de l'objet qui génèrera ces valeurs, autour d'une moyenne à 0 et avec un écart type de 200
#define N 20                            // Nombre de valeurs prises en compte pour la moyenne glissante
float HistoriqueSignal[N];              //Tableau utilisé comme buffer circulaire et destiné à stocker les N dernières valeurs du signal 
byte Plus_ancienSignal=0;               //cet index repère la valeur la plus ancienne dans le buffer circulaire. 
float TotalBufferCirculaireSignal=0;    //stockage du total de toutes les valeurs du buffer.
float SignalFiltreMG;                   // variable pour stocker la valeur du signal filtré par moyenne glissante
float s,b,sb;                           // s pour signal, b pour bruit, et sb pour signal + bruit
float sbIIR, sbIIRprev=0;               // variables utilisées pour l'application du filtre IIR

void setup() {
Serial.begin(115200);
for(byte i = 0; i < N; i ++) HistoriqueSignal[i] = 0; // Initialisation de toutes les valeurs du buffer circulaire à zéro

for( float x=0; x<=10; x+=0.02) {   //Boucle for pour le calcul puis le traçage des courbes  
  s = sin(x)*1000;         // Calcul de la sinusoïde (le signal "pur").
  b = myGaussian.random(); // génération d'un bruit gaussien
  sb = s + b;              // Ajout du bruit au signal
  
  sbIIR = valeurFiltreeRII(sbIIRprev,sb,0.085); // application du filtre récursif RII, le dernier argument passé à cette fonction est le coefficient du filtre
  sbIIRprev = sbIIR;                           // on enregistre cette valeur filtrée, elle servira au prochain passage dans la boucle for (récursivité)

// Calcul du total glissant de toutes les valeurs du buffer circulaire, ce total est placé dans la variable TotalBufferCirculaireSignal. 
  TotalBufferCirculaireSignal = TotalBufferCirculaireSignal + sb - HistoriqueSignal[Plus_ancienSignal]; // On ajoute au total précédent la valeur actuelle et on soustrait la valeur la plus ancienne.
  HistoriqueSignal[Plus_ancienSignal] = sb; //Cette plus ancienne valeur n'est plus utile, on y stocke la nouvelle, donc la plus récente
  Plus_ancienSignal ++; // On passe à la "case" suivante
  if(Plus_ancienSignal == N) Plus_ancienSignal = 0; //Si la "dernière case" du buffer est atteinte, on continue de façon circulaire à la "première"
  SignalFiltreMG = (TotalBufferCirculaireSignal/N); //On calcule la valeur lissée du signal, c'est à dire la moyenne du buffer, et on l'affecte à la variable SignalFiltreMG

// Traçage des courbes sur le traceur série
  Serial.print(s);
  Serial.print(",");
  Serial.print(sb);
  Serial.print(",");
  Serial.print(sbIIR);
  Serial.print(",");
  Serial.println(SignalFiltreMG); 
 }
}
void loop() {
  // Boucle loop() vide
}

// Fonction de filtre à réponse impulsionnelle infinie
float valeurFiltreeRII (float valeurFiltreePrecedente, float valeurCourante , float coeffFiltrageRII){
  return valeurFiltreePrecedente  * (1-coeffFiltrageRII) + valeurCourante * coeffFiltrageRII ;
}
